

--[[
--     CHAT_MSG_LOOT
]]--

local chatMsgLootFrame = CreateFrame("Frame")
chatMsgLootFrame:RegisterEvent("CHAT_MSG_LOOT")
chatMsgLootFrame:SetScript("OnEvent", function(self, event, ...)
    --if event == "CHAT_MSG_LOOT" then 
    --[[ CHAT MSG LOOT ]]--

    local t = TRCLOOT

    local message, sender, language, channelString, target, flags, _, channelNumber, channelName, _, counter = ...

    -- If it wasn't you that looted, stop
    --if not sender == GetUnitName("player") then return end



    -- Captures:   1 person/You, 2 itemstring, 3 rest of string after final |r until '.'
    -- Can change 'loot' to 'item' to trigger on, e.g., extracting stuff from mail
    local loot_pattern = "(%S+) receives? loot:.*|cff%x+|H(.-)|h.*|r(.*)%.$"


    -- CHAT_MSG_LOOT handler
    local msg =  message
    local person, itemstring, remainder = msg:match(loot_pattern)
    local count = remainder and remainder:match(".*(x%d+)$")

    if not person then return end    -- "So-and-So selected Greed", etc, not actual looting

    -- Name might be colorized, remove the highlighting
    local p = person:match("|c%x%x%x%x%x%x%x%x(%S+)")
    person = p or person
    if person ~= UNIT_YOU then
        return
    end

    -- Store the id in aura_env so that we can use it in
    -- the BAG_UPDATE event handler
    local itemID = tonumber((select(2, strsplit(":", itemstring))))
    t.itemsLooted = t.itemsLooted or {}
    table.insert(t.itemsLooted, itemID)
end)





--[[
--     BAG_UPDATE
]]--

local itemIsTradable = function(bag, slot)

    local t = TRCLOOT
    -- Create a new (hidden) tooltip
    local tooltipName = "RavenCouncil_ItemScanningTooltip"
    t.scanTooltip = t.scanTooltip or CreateFrame("GameTooltip", tooltipName, UIParent, "GameTooltipTemplate")
    t.scanTooltip:SetOwner(UIParent, "ANCHOR_NONE")

    -- Scan it for the tradeable string
    t.scanTooltip:ClearLines()
    t.scanTooltip:SetBagItem(bag, slot)

    for n=1, t.scanTooltip:NumLines() do
        local line = _G[tooltipName.."TextLeft"..n]:GetText()
        if string.find(line, "You may trade this item with players that were also eligible to loot this item for the next") then
            return true
        end
    end

    return false

end


local bagUpdateFrame = CreateFrame("Frame")
bagUpdateFrame:RegisterEvent("BAG_UPDATE")
bagUpdateFrame:SetScript("OnEvent", function(self, event, ...)

    local t = TRCLOOT

    t.itemsLooted = t.itemsLooted or {}

    -- If there is an item looted but not yet found in bags
    if next(t.itemsLooted) then
        -- Scan bags for the item based on item id

        -- for every bag
        for bag=0, 5 do
            -- for everty item in the bag
            for slot=0, GetContainerNumSlots(bag) do
                -- For every item in the queue
                local n = 1 
                while n <= #t.itemsLooted do
                    -- If we found an item
                    if t.itemsLooted[n] == GetContainerItemID(bag, slot) then
                        -- And if it is tradable
                        if itemIsTradable(bag, slot) then
                            -- Offer it to the rest of the raid
                            local hyperlink = GetContainerItemLink(bag, slot)
                            t.offerLoot(hyperlink)
                        end
                        -- Remove it from the list of items to look for in bags
                        table.remove(t.itemsLooted, n)
                    else
                        n = n+1
                    end
                end
            end    
        end

    end
end)




--[[
--     CHAT_MSG_ADDON
]]--

local chatMsgAddonFrame = CreateFrame("Frame")
chatMsgAddonFrame:RegisterEvent("CHAT_MSG_ADDON")
chatMsgAddonFrame:SetScript("OnEvent", function(self, event, ...)
    local t = TRCLOOT

    TRCLOOT.messages = TRCLOOT.messages or {}
    local messages = TRCLOOT.messages


    local prefix, msg, channel, sender = ...
    -- If msg for our addon
    if prefix == t.ADDON_MESSAGE_PREFIX then

        -- Obtain and remove prefix from msg
        local prefix = msg:sub(1,1)
        msg = msg:sub(2)

        -- Append message to current msg
        messages[sender] = (messages[sender] or "")..msg

        --If this is the last chunk, process it.
        if prefix == t.MSG_DONE then

            msg = messages[sender]
            -- Set to nil for next message
            messages[sender] = nil

            -- Deserialize the msg
            t.libS = t.libS or LibStub:GetLibrary("AceSerializer-3.0")

            local success, data = t.libS:Deserialize(msg)
            if not success then
                print("TRCLOOT: Something went wrong deserializing :(")
                print("Sender: "..sender)
                print("String: "..msg)

                return
            end

            -- Process the msg
            if data.type == t.OFFER then
                local hyperlink = data.hyperlink

                t.storeItem(hyperlink, sender)

                local specInfo = t.getVar("specs")
                for n=1, GetNumSpecializations() do
                    if specInfo[n]["enable"] then
                        t.calculateUpgradeScore(hyperlink, sender, n)
                    end
                end
                t.drawReports()
            elseif data.type == t.REPORT then
                data.sender = sender
                t.storeReport(data)
                t.drawReports()
            elseif data.type == t.VERSION_REQUEST then
                t.respondToVersionRequest(sender)
            elseif data.type == t.VERSION_RESPONSE then
                t.versionInfo = t.versionInfo or {}
                t.versionInfo[Ambiguate(sender, "short")] = data.versionNumber
            end

        end


    end
end)




--[[
--     ITEM INFO RECEIVED
]]--

local itemInfoReceivedFrame = CreateFrame("Frame")
itemInfoReceivedFrame:RegisterEvent("GET_ITEM_INFO_RECEIVED")
itemInfoReceivedFrame:SetScript("OnEvent", function(self, event, ...)
    --elseif event == "GET_ITEM_INFO_RECEIVED" then

    --[[ ITEM INFO RECEIVED ]]-- 
    -- GetItemInfo will return nil if the item is not currently loaded in memory.
    -- Whenever the item gets loaded, GET_ITEM_INFO_RECEIVED event will be thrown.
    -- This piece of code responds to that event by checking if it was the item 
    -- we were trying to call GetItemInfo() on, and if so calls a callback function.

    local t = TRCLOOT
    -- If we were waiting for items to load, loop over them
    if t.itemInfoQueue then
        local n = 1
        while n <= #t.itemInfoQueue do
        --for n, item in pairs(t.itemInfoQueue) do
            local item = t.itemInfoQueue[n]


            -- Deeply ashamed by this code, but hey, it works.
            local itemToCheck
            local itemToCheck2
            if item.callback == t.calculateUpgradeScoreCallback then
                itemToCheck = item.item
            else
                itemToCheck = item.equipedItem
                itemToCheck2 = item.equipedItem2
            end


            if GetItemInfo(itemToCheck) and
                ((not itemToCheck2) or (itemToCheck2 and GetItemInfo(itemToCheck2))) then


                -- Shame intensifies.
                if item.callback == t.calculateUpgradeScoreCallback then
                    item.callback(item.item, item.origin, item.spec)
                elseif item.callback == t.calculateUpgradeScoreCallback2 then
                    item.callback(item.item, item.equipedItem, item.equipedItem2, item.origin, item.spec)
                end

                table.remove(t.itemInfoQueue, n)
            else
                n = n + 1
            end
        end
    end
end)





--[[
--     ADDON LOADED
]]--

local addonLoadedFrame = CreateFrame("Frame")
addonLoadedFrame:RegisterEvent("ADDON_LOADED")
addonLoadedFrame:SetScript("OnEvent", function(self, event, addon)
    local t = TRCLOOT

    if addon == t.ADDON_NAME then
        if RavenLoot_svc == nil then
            RavenLoot_svc = {}
        end

        t.svc = RavenLoot_svc


        -- Set default stat weights if no stat weigths
        if not t.svc.statWeights then
            t.svc.defaultWeights = true
            t.svc.statWeights = t.defaultStatWeights
        end


    end


end)

--[[
--     PLAYER ENTERIGN WORLD
]]--

local playerEnteringWorldFrame = CreateFrame("Frame")
playerEnteringWorldFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
playerEnteringWorldFrame:SetScript("OnEvent", function(self, event, addon)

    local t = TRCLOOT
    local specs = t.getVar("specs")
    for n=1, GetNumSpecializations() do
        -- Load primary stats
        local primaryCode = select(7, GetSpecializationInfo(n))

        local primaryStat
        if primaryCode == LE_UNIT_STAT_STRENGTH then
            primaryStat = "ITEM_MOD_STRENGTH_SHORT"
        elseif primaryCode == LE_UNIT_STAT_INTELLECT then
            primaryStat = "ITEM_MOD_INTELLECT_SHORT"
        elseif primaryCode == LE_UNIT_STAT_AGILITY then
            primaryStat = "ITEM_MOD_AGILITY_SHORT"
        else
            print(t.MSG_PREFIX.."Unknown primary stat "..primaryCode..". Please report this issue to the author.")
            return
        end

        specs[n].primary = primaryStat




        -- Set equipmentSet
        if not specs[n].equipmentSet then
            specs[n].equipmentSet = GetEquipmentSetInfo(1)
        end
    end
end)





























